import json
import os
import random
from datetime import date

import numpy as np

from tqsdk import TqApi, TqSim, TqBacktest, utils, TargetPosScheduler
from tqsdk.algorithm import Twap, twap_table
from tqsdk.test.base_testcase import TQBaseTestcase
from tqsdk.test.test_chan_helper import set_test_script


class Unit_test(TQBaseTestcase):
    '''
     回测测试example

     注：
    1. 在本地运行测试用例前需设置运行环境变量(Environment variables), 保证api中dict及set等类型的数据序列在每次运行时元素顺序一致: PYTHONHASHSEED=32
    2. 若测试用例中调用了会使用uuid的功能函数时（如insert_order()会使用uuid生成order_id）,
        则：在生成script文件时及测试用例中都需设置 utils.RD = random.Random(x), 以保证两次生成的uuid一致, x取值范围为0-2^32
    3. 对盘中的测试用例（即非回测）：因为TqSim模拟交易 Order 的 insert_date_time 和 Trade 的 trade_date_time 不是固定值，所以改为判断范围。
        盘中时：self.assertAlmostEqual(1575292560005832000 / 1e9, order1.insert_date_time / 1e9, places=1)
        回测时：self.assertEqual(1575291600000000000, order1.insert_date_time)
    '''

    def setUp(self):
        super(Unit_test, self).setUp()

    def tearDown(self):
        super(Unit_test, self).tearDown()

    def test_twap(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        md_url = "wss://backtest.shinnytech.com/t/nfmd/front/mobile"
        set_test_script(os.path.join(dir_path, "examples_log_file", "test_algorithm_twap.script.lzma"))

        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(account=sim, _md_url=md_url, auth="tianqin,tianqin",
                    backtest=TqBacktest(start_dt=date(2021, 1, 13), end_dt=date(2021, 1, 14)))
        ticks = api.get_tick_serial("CZCE.MA105")
        quote = api.get_quote("CZCE.MA105")
        while quote.datetime <= "2021-01-12 21:00:26.000000":
            api.wait_update()
        order = api.insert_order("CZCE.MA105", "SELL", "OPEN", 200, quote.bid_price1)
        while order.status == "ALIVE":
            api.wait_update()
        # 设置twap任务参数
        target_twap = Twap(api, "CZCE.MA105", "BUY", "CLOSE", 152, 600, 1, 5)
        # 启动循环
        while not target_twap.is_finished():
            api.wait_update()
        sim.set_margin("CZCE.MA105", 1621.9)
        api.close()

        self.assertAlmostEqual(2347.940789473684, target_twap.average_trade_price)
        self.assertEqual(152, sum([t["volume"] for t in target_twap.trades if t["offset"] == "CLOSE"]))
        self.assertTrue(all([t["offset"] == "CLOSE" for t in target_twap.trades]))

        for k, v in sim.trade_log.items():
            trades = v["trades"]
        self.assertEqual(152, sum([t["volume"] for t in trades if t["offset"] == "CLOSE"]))
        self.assertEqual(200, sum([t["volume"] for t in trades if t["offset"] == "OPEN"]))

        self.assertEqual(48, api.get_position("CZCE.MA105", sim)["volume_short_today"])
        self.assertEqual(48, api.get_position("CZCE.MA105", sim)["volume_short"])

        trade_log = json.dumps(sim.trade_log)
        # print(trade_log)  # 用于生成测试用例except数据
        excepted_trade_log = '{"2021-01-13": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e|200", "exchange_trade_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e|200", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "SELL", "offset": "OPEN", "price": 2349.0, "volume": 200, "trade_date_time": 1610456426000001000, "commission": 400.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd", "trade_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd|3", "exchange_trade_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456434000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad", "trade_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad|3", "exchange_trade_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456440000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_285330653ed198e9863cebef04be3422", "trade_id": "PYSDK_target_285330653ed198e9863cebef04be3422|3", "exchange_trade_id": "PYSDK_target_285330653ed198e9863cebef04be3422|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456449000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_1f2c66dc14fa46c55107595ef3018e40", "trade_id": "PYSDK_target_1f2c66dc14fa46c55107595ef3018e40|3", "exchange_trade_id": "PYSDK_target_1f2c66dc14fa46c55107595ef3018e40|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 3, "trade_date_time": 1610456470000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_fe177771e0f39e520fae84924547e765", "trade_id": "PYSDK_target_fe177771e0f39e520fae84924547e765|1", "exchange_trade_id": "PYSDK_target_fe177771e0f39e520fae84924547e765|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 1, "trade_date_time": 1610456475000000000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e", "trade_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e|3", "exchange_trade_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456494000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786", "trade_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786|3", "exchange_trade_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456499000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e", "trade_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e|3", "exchange_trade_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456519000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578", "trade_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578|2", "exchange_trade_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 2, "trade_date_time": 1610456525000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff", "trade_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff|3", "exchange_trade_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456534000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426", "trade_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426|3", "exchange_trade_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 3, "trade_date_time": 1610456551000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0", "trade_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0|3", "exchange_trade_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 3, "trade_date_time": 1610456555000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736", "trade_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736|5", "exchange_trade_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610456569000001000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d", "trade_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d|3", "exchange_trade_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456588000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f", "trade_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f|4", "exchange_trade_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 4, "trade_date_time": 1610456601000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5", "trade_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5|2", "exchange_trade_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456613000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c", "trade_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c|2", "exchange_trade_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456623000001000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9", "trade_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9|3", "exchange_trade_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 3, "trade_date_time": 1610456641000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b", "trade_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b|4", "exchange_trade_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 4, "trade_date_time": 1610456643000001000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8", "trade_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8|3", "exchange_trade_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456662000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e5ac6e61d4f36f94a597dd1bee32a308", "trade_id": "PYSDK_target_e5ac6e61d4f36f94a597dd1bee32a308|4", "exchange_trade_id": "PYSDK_target_e5ac6e61d4f36f94a597dd1bee32a308|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 4, "trade_date_time": 1610456663000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb", "trade_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb|3", "exchange_trade_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2352.0, "volume": 3, "trade_date_time": 1610456683000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e54902a9c8ca32874eb13e88974dff0e", "trade_id": "PYSDK_target_e54902a9c8ca32874eb13e88974dff0e|2", "exchange_trade_id": "PYSDK_target_e54902a9c8ca32874eb13e88974dff0e|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 2, "trade_date_time": 1610456693000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_56d25122d570dd7da00390157a3c4725", "trade_id": "PYSDK_target_56d25122d570dd7da00390157a3c4725|3", "exchange_trade_id": "PYSDK_target_56d25122d570dd7da00390157a3c4725|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 3, "trade_date_time": 1610456697000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868", "trade_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868|4", "exchange_trade_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456718000001000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b2dff7a7f9bf5bd6a888d4cc1a82d1c3", "trade_id": "PYSDK_target_b2dff7a7f9bf5bd6a888d4cc1a82d1c3|2", "exchange_trade_id": "PYSDK_target_b2dff7a7f9bf5bd6a888d4cc1a82d1c3|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 2, "trade_date_time": 1610456721000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_5a6219a1f48743d5fcc920ce79e4bbfc", "trade_id": "PYSDK_target_5a6219a1f48743d5fcc920ce79e4bbfc|3", "exchange_trade_id": "PYSDK_target_5a6219a1f48743d5fcc920ce79e4bbfc|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456745000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a", "trade_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a|2", "exchange_trade_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 2, "trade_date_time": 1610456756000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2", "trade_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2|3", "exchange_trade_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456769000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d3a1a0e7e3855179fa80137481ef1c07", "trade_id": "PYSDK_target_d3a1a0e7e3855179fa80137481ef1c07|4", "exchange_trade_id": "PYSDK_target_d3a1a0e7e3855179fa80137481ef1c07|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456783000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235", "trade_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235|3", "exchange_trade_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456789000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_9f94a80839e3854cd8070044124d54d1", "trade_id": "PYSDK_target_9f94a80839e3854cd8070044124d54d1|2", "exchange_trade_id": "PYSDK_target_9f94a80839e3854cd8070044124d54d1|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456798000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45", "trade_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45|4", "exchange_trade_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456818000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e", "trade_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e|3", "exchange_trade_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456820000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_63721f64db9fbff04c520178fb8930db", "trade_id": "PYSDK_target_63721f64db9fbff04c520178fb8930db|4", "exchange_trade_id": "PYSDK_target_63721f64db9fbff04c520178fb8930db|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 4, "trade_date_time": 1610456836000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f99231987e62c170698c49232c40e270", "trade_id": "PYSDK_target_f99231987e62c170698c49232c40e270|3", "exchange_trade_id": "PYSDK_target_f99231987e62c170698c49232c40e270|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456850000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78", "trade_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78|5", "exchange_trade_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 5, "trade_date_time": 1610456868000000000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e312dffb7a862cef0498119036f72908", "trade_id": "PYSDK_target_e312dffb7a862cef0498119036f72908|1", "exchange_trade_id": "PYSDK_target_e312dffb7a862cef0498119036f72908|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 1, "trade_date_time": 1610456872000000000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666", "trade_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666|3", "exchange_trade_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456881000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d7874408ed7d0b4ca0a52b2df32ce83c", "trade_id": "PYSDK_target_d7874408ed7d0b4ca0a52b2df32ce83c|4", "exchange_trade_id": "PYSDK_target_d7874408ed7d0b4ca0a52b2df32ce83c|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 4, "trade_date_time": 1610456903000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_9ebfefa0c01288b6523aabc668d90aa1", "trade_id": "PYSDK_target_9ebfefa0c01288b6523aabc668d90aa1|2", "exchange_trade_id": "PYSDK_target_9ebfefa0c01288b6523aabc668d90aa1|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 2, "trade_date_time": 1610456908000001000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d", "trade_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d|1", "exchange_trade_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 1, "trade_date_time": 1610456923000001000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_ea1cea7aacfa72d54982741c9be0fd55", "trade_id": "PYSDK_target_ea1cea7aacfa72d54982741c9be0fd55|2", "exchange_trade_id": "PYSDK_target_ea1cea7aacfa72d54982741c9be0fd55|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 2, "trade_date_time": 1610456940000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552", "trade_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552|3", "exchange_trade_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 3, "trade_date_time": 1610456940000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c", "trade_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c|3", "exchange_trade_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2344.0, "volume": 3, "trade_date_time": 1610456955000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5", "trade_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5|3", "exchange_trade_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2343.0, "volume": 3, "trade_date_time": 1610456973000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4", "trade_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4|3", "exchange_trade_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2344.0, "volume": 3, "trade_date_time": 1610456978000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa", "trade_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa|5", "exchange_trade_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610456996000000000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93", "trade_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93|4", "exchange_trade_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 4, "trade_date_time": 1610457013000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e83253e4269872bca91f899894f90725", "trade_id": "PYSDK_target_e83253e4269872bca91f899894f90725|5", "exchange_trade_id": "PYSDK_target_e83253e4269872bca91f899894f90725|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610457025000000000, "commission": 10.0}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10003306.0, "available": 9925454.799999986, "float_profit": 2400.0, "position_profit": 2400.0, "close_profit": 1610.0, "frozen_margin": 0.0, "margin": 77851.20000000001, "frozen_commission": 0.0, "commission": 704.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.0077825470899320695, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"CZCE.MA105": {"exchange_id": "CZCE", "instrument_id": "MA105", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 48, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 48, "volume_short_his": 0, "volume_short": 48, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 2349.0, "open_cost_long": 0.0, "open_cost_short": 1127520.0, "position_price_long": NaN, "position_price_short": 2349.0, "position_cost_long": 0.0, "position_cost_short": 1127520.0, "float_profit_long": 0.0, "float_profit_short": 2400.0, "float_profit": 2400.0, "position_profit_long": 0.0, "position_profit_short": 2400.0, "position_profit": 2400.0, "margin_long": 0.0, "margin_short": 77851.20000000001, "margin": 77851.20000000001, "last_price": 2344.0, "underlying_last_price": NaN, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0, "future_margin": 1621.9}}}}'
        self.assertEqual(trade_log, excepted_trade_log, msg="交易记录及收盘时的权益及持仓: 信息错误")

    def test_twap2(self):
        dir_path = os.path.dirname(os.path.realpath(__file__))
        md_url = "wss://backtest.shinnytech.com/t/nfmd/front/mobile"
        set_test_script(os.path.join(dir_path, "examples_log_file", "test_algorithm_twap.script.lzma"))

        utils.RD = random.Random(4)
        sim = TqSim()
        api = TqApi(account=sim, _md_url=md_url, auth="tianqin,tianqin",
                    backtest=TqBacktest(start_dt=date(2021, 1, 13), end_dt=date(2021, 1, 14)))
        ticks = api.get_tick_serial("CZCE.MA105")
        quote = api.get_quote("CZCE.MA105")
        while quote.datetime <= "2021-01-12 21:00:26.000000":
            api.wait_update()
        order = api.insert_order("CZCE.MA105", "SELL", "OPEN", 200, quote.bid_price1)
        while order.status == "ALIVE":
            api.wait_update()
        # 设置twap任务参数
        time_table = twap_table(api, "CZCE.MA105", -48, 600, 1, 5)
        target_pos_sch = TargetPosScheduler(api, "CZCE.MA105", time_table)
        # 启动循环
        while not target_pos_sch.is_finished():
            api.wait_update()
        sim.set_margin("CZCE.MA105", 1621.9)
        api.close()

        average_trade_price = sum(target_pos_sch.trades_df['price'] * target_pos_sch.trades_df['volume']) / sum(target_pos_sch.trades_df['volume'])
        self.assertAlmostEqual(2347.9539473684213, average_trade_price)
        self.assertEqual(152, target_pos_sch.trades_df['volume'].sum())
        self.assertTrue(np.where(target_pos_sch.trades_df['offset'] == "CLOSE", True, False).all())

        for k, v in sim.trade_log.items():
            trades = v["trades"]
        self.assertEqual(152, sum([t["volume"] for t in trades if t["offset"] == "CLOSE"]))
        self.assertEqual(200, sum([t["volume"] for t in trades if t["offset"] == "OPEN"]))

        self.assertEqual(48, api.get_position("CZCE.MA105", sim)["volume_short_today"])
        self.assertEqual(48, api.get_position("CZCE.MA105", sim)["volume_short"])

        trade_log = json.dumps(sim.trade_log)
        # print(trade_log)  # 用于生成测试用例except数据
        excepted_trade_log = '{"2021-01-13": {"trades": [{"user_id": "TQSIM", "order_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e", "trade_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e|200", "exchange_trade_id": "PYSDK_insert_c79d679346d4ac7a5c3902b38963dc6e|200", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "SELL", "offset": "OPEN", "price": 2349.0, "volume": 200, "trade_date_time": 1610456426000001000, "commission": 400.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd", "trade_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd|3", "exchange_trade_id": "PYSDK_target_ca5a6e3165dd409a29cf50740c009fdd|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456434000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad", "trade_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad|3", "exchange_trade_id": "PYSDK_target_8c110e419ee4509cae421df494f0c9ad|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456440000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_4328705489dad801f5944f0fb6897eda", "trade_id": "PYSDK_target_4328705489dad801f5944f0fb6897eda|3", "exchange_trade_id": "PYSDK_target_4328705489dad801f5944f0fb6897eda|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456458000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_0820b40b349ae3a218c56efac055bbbe", "trade_id": "PYSDK_target_0820b40b349ae3a218c56efac055bbbe|3", "exchange_trade_id": "PYSDK_target_0820b40b349ae3a218c56efac055bbbe|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 3, "trade_date_time": 1610456470000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_fe177771e0f39e520fae84924547e765", "trade_id": "PYSDK_target_fe177771e0f39e520fae84924547e765|1", "exchange_trade_id": "PYSDK_target_fe177771e0f39e520fae84924547e765|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 1, "trade_date_time": 1610456484000000000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e", "trade_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e|3", "exchange_trade_id": "PYSDK_target_bb3835092c1d71a8b3b52c42a1aa036e|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456494000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786", "trade_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786|3", "exchange_trade_id": "PYSDK_target_f3de70f26a3ace5da7b9d5ab2631c786|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456499000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e", "trade_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e|3", "exchange_trade_id": "PYSDK_target_b82cca76b58c1f17e0e4206c2147494e|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456519000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578", "trade_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578|2", "exchange_trade_id": "PYSDK_target_8d6710a7e16b31fdb944d47700aa4578|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 2, "trade_date_time": 1610456525000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff", "trade_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff|3", "exchange_trade_id": "PYSDK_target_2779bdf86973a73e80a7104827e14aff|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456534000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426", "trade_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426|3", "exchange_trade_id": "PYSDK_target_2b04527a603f852a115b6b4781f1a426|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 3, "trade_date_time": 1610456551000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0", "trade_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0|3", "exchange_trade_id": "PYSDK_target_418dd631e3c0a00ddff1c6ca29d58ae0|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 3, "trade_date_time": 1610456555000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736", "trade_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736|5", "exchange_trade_id": "PYSDK_target_aea0c5e6898ff54d642dc72483cc5736|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610456569000001000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d", "trade_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d|3", "exchange_trade_id": "PYSDK_target_6c4f61950fda0caa6305b03b2b96664d|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456588000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f", "trade_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f|4", "exchange_trade_id": "PYSDK_target_f5c8bf794dd2c99504dd27b6ad1c476f|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 4, "trade_date_time": 1610456601000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5", "trade_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5|2", "exchange_trade_id": "PYSDK_target_8db3d5a93b34e5e83b658bd203efb5b5|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456613000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c", "trade_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c|2", "exchange_trade_id": "PYSDK_target_d3535381e2ae25c52f6265e28840cc2c|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456623000001000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9", "trade_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9|3", "exchange_trade_id": "PYSDK_target_efe751e9aefe694debd5b306de9b7ed9|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 3, "trade_date_time": 1610456641000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b", "trade_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b|4", "exchange_trade_id": "PYSDK_target_9654825487c337ebd5713c5a2c2dd56b|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 4, "trade_date_time": 1610456646000001000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8", "trade_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8|3", "exchange_trade_id": "PYSDK_target_4e1e30495b565cc2e9679cd3e7303dd8|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456662000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb", "trade_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb|4", "exchange_trade_id": "PYSDK_target_a9759216cd621a60dba384c09437b0eb|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 4, "trade_date_time": 1610456673000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e4414ee71553c650d58c4282ceac6f1f", "trade_id": "PYSDK_target_e4414ee71553c650d58c4282ceac6f1f|3", "exchange_trade_id": "PYSDK_target_e4414ee71553c650d58c4282ceac6f1f|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2352.0, "volume": 3, "trade_date_time": 1610456683000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868", "trade_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868|2", "exchange_trade_id": "PYSDK_target_e21d9f3d50e7b0e1e4a8765d3c6cc868|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 2, "trade_date_time": 1610456693000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_86ef8378528571389acc664d09ca2aa8", "trade_id": "PYSDK_target_86ef8378528571389acc664d09ca2aa8|3", "exchange_trade_id": "PYSDK_target_86ef8378528571389acc664d09ca2aa8|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2351.0, "volume": 3, "trade_date_time": 1610456705000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b6a893b72d58c1ee14dde0b7eda5697d", "trade_id": "PYSDK_target_b6a893b72d58c1ee14dde0b7eda5697d|4", "exchange_trade_id": "PYSDK_target_b6a893b72d58c1ee14dde0b7eda5697d|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456718000001000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a", "trade_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a|2", "exchange_trade_id": "PYSDK_target_3ff7ce03b6048259990e3052d3c0a74a|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 2, "trade_date_time": 1610456725000001000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2", "trade_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2|3", "exchange_trade_id": "PYSDK_target_e890f8559a0a2dfcc8161be5561dc2b2|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456745000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235", "trade_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235|2", "exchange_trade_id": "PYSDK_target_003b13d5d6c16f48826188ead32bd235|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 2, "trade_date_time": 1610456756000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_04ad1028f4579f329f4c1d3946335508", "trade_id": "PYSDK_target_04ad1028f4579f329f4c1d3946335508|3", "exchange_trade_id": "PYSDK_target_04ad1028f4579f329f4c1d3946335508|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 3, "trade_date_time": 1610456769000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2515c174b29b35197a638e1d02b08fb3", "trade_id": "PYSDK_target_2515c174b29b35197a638e1d02b08fb3|4", "exchange_trade_id": "PYSDK_target_2515c174b29b35197a638e1d02b08fb3|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456783000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45", "trade_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45|3", "exchange_trade_id": "PYSDK_target_e3e1ae853fcc5a7f5c1e1f6236907e45|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456789000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e", "trade_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e|2", "exchange_trade_id": "PYSDK_target_d0dff051584f878fbf969bfcbd09713e|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 2, "trade_date_time": 1610456798000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_dcdc3f5882b886d5c108d4be105d1701", "trade_id": "PYSDK_target_dcdc3f5882b886d5c108d4be105d1701|4", "exchange_trade_id": "PYSDK_target_dcdc3f5882b886d5c108d4be105d1701|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2350.0, "volume": 4, "trade_date_time": 1610456818000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_b45e50b035242ef1c9f11a64486a88d5", "trade_id": "PYSDK_target_b45e50b035242ef1c9f11a64486a88d5|3", "exchange_trade_id": "PYSDK_target_b45e50b035242ef1c9f11a64486a88d5|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 3, "trade_date_time": 1610456830000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78", "trade_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78|4", "exchange_trade_id": "PYSDK_target_2cc43ab052646e876ff994fcb4108c78|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 4, "trade_date_time": 1610456836000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666", "trade_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666|3", "exchange_trade_id": "PYSDK_target_46a4565ff8c42f8dee8c9bb36fed5666|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456850000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d", "trade_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d|5", "exchange_trade_id": "PYSDK_target_82d8c1924796a2f14188715e35cd654d|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2349.0, "volume": 5, "trade_date_time": 1610456868000000000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_43796067672d879b07b37f1e162c769a", "trade_id": "PYSDK_target_43796067672d879b07b37f1e162c769a|1", "exchange_trade_id": "PYSDK_target_43796067672d879b07b37f1e162c769a|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2348.0, "volume": 1, "trade_date_time": 1610456872000000000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552", "trade_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552|3", "exchange_trade_id": "PYSDK_target_c21e8fa69ce8815c21e51108a439e552|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 3, "trade_date_time": 1610456892000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c", "trade_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c|4", "exchange_trade_id": "PYSDK_target_1795d2109ec5b1272a7e5e12c841310c|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2347.0, "volume": 4, "trade_date_time": 1610456903000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_eedd3787ef34bbca362ba7c0b3762444", "trade_id": "PYSDK_target_eedd3787ef34bbca362ba7c0b3762444|2", "exchange_trade_id": "PYSDK_target_eedd3787ef34bbca362ba7c0b3762444|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 2, "trade_date_time": 1610456908000001000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5", "trade_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5|1", "exchange_trade_id": "PYSDK_target_e3fb52783753aaf9ab33a1e740ebf5e5|1", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 1, "trade_date_time": 1610456923000001000, "commission": 2.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4", "trade_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4|2", "exchange_trade_id": "PYSDK_target_74a3e1700209526b572ad3d6a53f2bf4|2", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2346.0, "volume": 2, "trade_date_time": 1610456940000000000, "commission": 4.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa", "trade_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa|3", "exchange_trade_id": "PYSDK_target_d7df0cbd84e23c5d3e81529517f06baa|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2344.0, "volume": 3, "trade_date_time": 1610456954000000000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93", "trade_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93|3", "exchange_trade_id": "PYSDK_target_f8ad5007cc2b9f691d2f5eb718673e93|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2343.0, "volume": 3, "trade_date_time": 1610456965000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_e83253e4269872bca91f899894f90725", "trade_id": "PYSDK_target_e83253e4269872bca91f899894f90725|3", "exchange_trade_id": "PYSDK_target_e83253e4269872bca91f899894f90725|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2343.0, "volume": 3, "trade_date_time": 1610456973000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_c444429ac1849bd106f7e0f7bf373098", "trade_id": "PYSDK_target_c444429ac1849bd106f7e0f7bf373098|3", "exchange_trade_id": "PYSDK_target_c444429ac1849bd106f7e0f7bf373098|3", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 3, "trade_date_time": 1610456984000001000, "commission": 6.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_530d03ee97fa613b08ec43e2434f3e0e", "trade_id": "PYSDK_target_530d03ee97fa613b08ec43e2434f3e0e|5", "exchange_trade_id": "PYSDK_target_530d03ee97fa613b08ec43e2434f3e0e|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610456996000000000, "commission": 10.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_fba0dd3ebc671f7f54010793a1a1afc4", "trade_id": "PYSDK_target_fba0dd3ebc671f7f54010793a1a1afc4|4", "exchange_trade_id": "PYSDK_target_fba0dd3ebc671f7f54010793a1a1afc4|4", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 4, "trade_date_time": 1610457013000000000, "commission": 8.0}, {"user_id": "TQSIM", "order_id": "PYSDK_target_2bfffb392990298bfd77cd71fc7d777e", "trade_id": "PYSDK_target_2bfffb392990298bfd77cd71fc7d777e|5", "exchange_trade_id": "PYSDK_target_2bfffb392990298bfd77cd71fc7d777e|5", "exchange_id": "CZCE", "instrument_id": "MA105", "direction": "BUY", "offset": "CLOSE", "price": 2345.0, "volume": 5, "trade_date_time": 1610457025000000000, "commission": 10.0}], "account": {"currency": "CNY", "pre_balance": 10000000.0, "static_balance": 10000000.0, "balance": 10003286.0, "available": 9925434.799999986, "float_profit": 2400.0, "position_profit": 2400.0, "close_profit": 1590.0, "frozen_margin": 0.0, "margin": 77851.20000000001, "frozen_commission": 0.0, "commission": 704.0, "frozen_premium": 0.0, "premium": 0.0, "deposit": 0.0, "withdraw": 0.0, "risk_ratio": 0.007782562649913239, "market_value": 0.0, "ctp_balance": NaN, "ctp_available": NaN}, "positions": {"CZCE.MA105": {"exchange_id": "CZCE", "instrument_id": "MA105", "pos_long_his": 0, "pos_long_today": 0, "pos_short_his": 0, "pos_short_today": 48, "volume_long_today": 0, "volume_long_his": 0, "volume_long": 0, "volume_long_frozen_today": 0, "volume_long_frozen_his": 0, "volume_long_frozen": 0, "volume_short_today": 48, "volume_short_his": 0, "volume_short": 48, "volume_short_frozen_today": 0, "volume_short_frozen_his": 0, "volume_short_frozen": 0, "open_price_long": NaN, "open_price_short": 2349.0, "open_cost_long": 0.0, "open_cost_short": 1127520.0, "position_price_long": NaN, "position_price_short": 2349.0, "position_cost_long": 0.0, "position_cost_short": 1127520.0, "float_profit_long": 0.0, "float_profit_short": 2400.0, "float_profit": 2400.0, "position_profit_long": 0.0, "position_profit_short": 2400.0, "position_profit": 2400.0, "margin_long": 0.0, "margin_short": 77851.20000000001, "margin": 77851.20000000001, "last_price": 2344.0, "underlying_last_price": NaN, "market_value_long": 0.0, "market_value_short": 0.0, "market_value": 0.0, "future_margin": 1621.9}}}}'
        self.assertEqual(trade_log, excepted_trade_log, msg="交易记录及收盘时的权益及持仓: 信息错误")